/******************************************************************************/
/*                                                                            */
/* Module     : gvibBase.i                                                    */
/*                                                                            */
/* Description: This module is the mainline SWIG source file for the C code   */
/*              associated with the python Cursor class and which is used to  */
/*              interface with the Interbase API library.  This module        */
/*              implements some typemaps and the initialization code for the  */
/*              C Cursor object.  This module includes the other SWIG files   */
/*              that do the real work.                                        */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Copyright (c) 2000 Going Virtual, LLC. All rights reserved.                */
/*                                                                            */
/* This library is free software; you can redistribute it and/or              */
/* modify it under the terms of the GNU Lesser General Public                 */
/* License as published by the Free Software Foundation; either               */
/* version 2.1 of the License, or (at your option) any later version.         */
/*                                                                            */
/* This library is distributed in the hope that it will be useful,            */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of             */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU          */
/* Lesser General Public License for more details.                            */
/*                                                                            */
/* You should have received a copy of the GNU Lesser General Public           */
/* License along with this library; if not, write to the Free Software        */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  */
/*                                                                            */
/*                                                                            */
/* Contact Information:                                                       */
/*                                                                            */
/* Going Virtual, LLC                                                         */
/* 10800 Independence Pointe Pkwy                                             */
/* Matthews, NC 28105                                                         */
/* www.goingv.com                                                             */
/*                                                                            */
/* Robert E. Tierney                                                          */
/* bob.tierney@goingv.com                                                     */
/* 1 (704) 849-0731 x109                                                      */
/*                                                                            */
/******************************************************************************/
/* Change History                                                             */
/*                                                                            */
/* 07/31/2000 - Fixed bug in IBInitExceptions. Thanks to Brad Clemenets.      */
/*                                                                            */
/******************************************************************************/
/*
 * $Id: gvibBase.i,v 1.7 2000/08/09 13:43:40 btierney Exp $
 * $Revision: 1.7 $
 */
%module gvibBase

%{  /* Start SWIG-Free Section */

#include <string.h>
#include <time.h>
#include <Python.h>
#include <ibase.h>

/*----------------------------------------------------------------------------*/
/* The C Connection structure.                                                */
/*----------------------------------------------------------------------------*/
typedef struct
{
  PyObject_HEAD
  unsigned short page_size;
  isc_db_handle db_handle;                        /* Database handle          */
  isc_tr_handle tr_handle;                        /* Transaction handle       */
  ISC_STATUS status[20];                          /* Persistent status vector */
}
Connection;

/*----------------------------------------------------------------------------*/
/* The C Cursor structure.                                                    */
/*----------------------------------------------------------------------------*/
typedef struct
{
  PyObject_HEAD
  Connection *connect;                         /* DB handle, TR handle, etc.  */
  isc_stmt_handle stmt_handle;                 /* SQL statement handle.       */
  XSQLDA *isqlda;                              /* SQLDA for input parameters. */
  XSQLDA *osqlda;                              /* SQLDA for output columns.   */
  ISC_STATUS status[20];                       /* Persistent status vector.   */
}
Cursor;

/*----------------------------------------------------------------------------*/
/* Public Function Prototypes (called by python code)                         */
/*----------------------------------------------------------------------------*/
static PyObject *IBSetException       (ISC_STATUS *);
static PyObject *IBConnect            (char *, char *, char *, PyObject *, 
                                       PyObject *);
static void      IBConnectDeallocate  (Connection *);
static PyObject *IBConnectGetAttr     (Connection *, char *);
static PyObject *IBCursor             (Connection *, long);
static void      IBCursorDeallocate   (Cursor *);
static long      IBCursorReallocXDA   (Cursor *, char *);
static PyObject *IBCursorGetAttr      (Cursor *, char *);
static int       IBCursorSetAttr      (Cursor *, char *, PyObject *);
static PyObject *IBCursorGetRow       (Cursor *);
static PyObject *IBCursorGetColDesc   (Cursor *);
static PyObject *IBCursorBindParm     (Cursor *, long, PyObject *);
static long      IBCursorBindCols     (Cursor *);
static long      IBCursorFreeCols     (Cursor *, char *);
static PyObject *IBSetDateTimeCallback(PyObject *);

/*----------------------------------------------------------------------------*/
/* Private Function Prototypes                                                */
/*----------------------------------------------------------------------------*/
static int  initExceptions           (void);
static void setBadParmException      (PyObject *, char *, long);
static void setBadParmIndexException (long, short);
static void setOverflowException     (long);

static short     getPrecision (Cursor *, char *);
static PyObject *getBlob      (Cursor *, ISC_QUAD *);
static short     putBlob      (Cursor *, ISC_QUAD *, PyObject *);

static short bindParmVarying   (XSQLVAR *, PyObject *, long);
static short bindParmText      (XSQLVAR *, PyObject *, long);
static short bindParmShort     (XSQLVAR *, PyObject *, long);
static short bindParmLong      (XSQLVAR *, PyObject *, long);
static short bindParmFloat     (XSQLVAR *, PyObject *, long);
static short bindParmDouble    (XSQLVAR *, PyObject *, long);
static short bindParmTimestamp (XSQLVAR *, PyObject *, long);
static short bindParmTime      (XSQLVAR *, PyObject *, long);
static short bindParmDate      (XSQLVAR *, PyObject *, long);
static short bindParmInt64     (XSQLVAR *, PyObject *, long);
static short bindParmFixedShort(XSQLVAR *, PyObject *, long);
static short bindParmFixedLong (XSQLVAR *, PyObject *, long);
static short bindParmFixedInt64(XSQLVAR *, PyObject *, long);
static short bindParmBlob      (XSQLVAR *, PyObject *, long, Cursor *);

/*----------------------------------------------------------------------------*/
/* Macro definitions                                                          */
/*----------------------------------------------------------------------------*/
#define DB_ERROR(sv) (sv[0] == 1 && sv[1])
#define IS_NULL     -1;
#define IS_NOT_NULL  0;

#define MIN(x, y) ((x) < (y) ? (x) : (y))
#define MAX(x, y) ((x) > (y) ? (x) : (y))

#define BLOB_SEGMENT_SIZE 1024

/*----------------------------------------------------------------------------*/
/* Private global variables                                                   */
/*----------------------------------------------------------------------------*/
static char     *separator  = " -- ";
static PyObject *msgTable   = NULL;
static PyObject *PyExcTable = NULL;
static PyObject *PyDateTime = NULL;

static PyObject *PyExc_Warning;
static PyObject *PyExc_Error;
static PyObject *PyExc_InterfaceError;
static PyObject *PyExc_DatabaseError;
static PyObject *PyExc_DataError;
static PyObject *PyExc_OperationalError;
static PyObject *PyExc_IntegrityError;
static PyObject *PyExc_InternalError;
static PyObject *PyExc_ProgrammingError;
static PyObject *PyExc_NotSupportedError;


%}  /* End SWIG-Free Section */

%include pointer.i
%include typemaps.i

/*----------------------------------------------------------------------------*/
/* Typemaps for parameter conversions.                                        */
/*----------------------------------------------------------------------------*/
%typemap(python, in) char *
{
  if (!PyString_Check($source))
    return NULL;

  if ($source == Py_None)
    $target = NULL;
  else
    $target = PyString_AsString($source);
}

%typemap(python, in) Connection *
{
  if ($source->ob_type != &ConnectionType)
  {
    PyErr_SetString(PyExc_TypeError, 
                    "Parameter must be a ConnectionType object!");
    return NULL;
  }

  $target = ((Connection *) $source);
}

%typemap(python, in) Cursor *
{
  if ($source->ob_type != &CursorType)
  {
    PyErr_SetString(PyExc_TypeError, 
                    "Parameter must be a CursorType object!");
    return NULL;
  }

  $target = ((Cursor *) $source);
}

%typemap(python, in) long
{
  if (!PyInt_Check($source))
    return NULL;

  $target = PyInt_AsLong($source);
}

/*----------------------------------------------------------------------------*/
/* Include the SWIG files that do the real work.                              */
/*----------------------------------------------------------------------------*/
%include "gvibExceptions.i"
%include "gvibConnect.i"
%include "gvibCursor.i"
%include "gvibBindParm.i"

/*----------------------------------------------------------------------------*/
/* Interbase API function prototypes.  These must be swigged.                 */
/*----------------------------------------------------------------------------*/
typedef long ISC_STATUS
typedef int  ISC_LONG

ISC_STATUS isc_database_info(
		ISC_STATUS *,
		isc_db_handle *,
		short,
		char *,
		short,
		char *);

ISC_STATUS isc_detach_database (
		ISC_STATUS *, 
		isc_db_handle *);

ISC_STATUS isc_commit_retaining (
		ISC_STATUS *, 
		isc_tr_handle *);

ISC_STATUS isc_dsql_allocate_statement (
		ISC_STATUS *, 
		isc_db_handle *,
		isc_stmt_handle *);

ISC_STATUS isc_dsql_describe (
		ISC_STATUS *, 
		isc_stmt_handle *, 
		unsigned short, 
		XSQLDA *);

ISC_STATUS isc_dsql_describe_bind (
		ISC_STATUS *, 
		isc_stmt_handle *, 
		unsigned short, 
		XSQLDA *);

ISC_STATUS isc_dsql_execute (
		ISC_STATUS *, 
		isc_tr_handle *, 
		isc_stmt_handle *, 
		unsigned short, 
		XSQLDA *);

ISC_STATUS isc_dsql_execute2 (
		ISC_STATUS *, 
		isc_tr_handle *, 
		isc_stmt_handle *, 
		unsigned short, 
		XSQLDA *,
		XSQLDA *);

ISC_STATUS isc_dsql_fetch (
		ISC_STATUS *, 
		isc_stmt_handle *, 
		unsigned short, 
		XSQLDA *);

ISC_STATUS isc_dsql_free_statement (
		ISC_STATUS *, 
		isc_stmt_handle *, 
		unsigned short);

ISC_STATUS isc_dsql_prepare (
		ISC_STATUS *, 
		isc_tr_handle *, 
		isc_stmt_handle *, 
		unsigned short, 
		char *, 
		unsigned short, 
		XSQLDA *);

ISC_STATUS isc_dsql_set_cursor_name (
		ISC_STATUS *, 
		isc_stmt_handle *, 
		char *, 
		unsigned short);

ISC_STATUS isc_dsql_sql_info (
		ISC_STATUS *, 
		isc_stmt_handle *, 
		unsigned short, 
		char *, 
		unsigned short, 
		char *);

ISC_STATUS isc_rollback_retaining (
		ISC_STATUS *, 
		isc_tr_handle *);

ISC_STATUS isc_rollback_transaction (
		ISC_STATUS *, 
		isc_tr_handle *);

ISC_LONG isc_vax_integer (
		char *,
		short);
                             
%inline
%{

/******************************************************************************/
/*                                                                            */
/* IBSetDateTimeCallback                                                      */
/*                                                                            */
/* This method is called from python to initialize the PyDateTime variable    */
/* with a python method.  This will allow the C code to call back into the    */
/* python code to instantiate a Date/Time/Timestamp object whenever one of    */
/* these types appear in a select call and we are fetching the data.          */
/*                                                                            */
/******************************************************************************/
static PyObject *IBSetDateTimeCallback(PyObject *fn)
{
  PyObject *tmp;

  if (PyArg_ParseTuple(fn, "O:IBSetDateTimeCallback", &tmp))
  {
    if (!PyCallable_Check(tmp))
      return NULL;

    Py_XINCREF(tmp);
    Py_XDECREF(PyDateTime);

    PyDateTime = tmp;

    Py_INCREF(Py_None);

    return Py_None;
  }

  return NULL;
}


/******************************************************************************/
/*                                                                            */
/* IBInitMessages                                                             */
/*                                                                            */
/* This function is called from the python code to initialize the static      */
/* message table variable.                                                    */
/*                                                                            */
/******************************************************************************/
static int IBInitMessages(PyObject * arg)
{
  if (PyArg_ParseTuple(arg, "O", &msgTable))
    return -1;
}


/******************************************************************************/
/*                                                                            */
/* IBInitExceptions                                                           */
/*                                                                            */
/* This function is called from the python code to initialize the static      */
/* exceptions variables.                                                      */
/*                                                                            */
/******************************************************************************/
static int IBInitExceptions (PyObject *args)
{
  int rc;

  rc = PyArg_ParseTuple(args, "OOOOOOOOOOO",
                           &PyExc_Warning,
			   &PyExc_Error, 
			   &PyExc_InterfaceError, 
			   &PyExc_DatabaseError, 
			   &PyExc_DataError, 
			   &PyExc_OperationalError, 
			   &PyExc_IntegrityError, 
			   &PyExc_InternalError, 
			   &PyExc_ProgrammingError, 
			   &PyExc_NotSupportedError, 
			   &PyExcTable); 
			   
  /*--------------------------------------------------------------------------*/
  /* The test should be 'if (!rc) instead of 'if (rc)'.  A zero is an error.  */
  /* 07/31/2000  Thanks to Brad Clements.                                     */
  /*--------------------------------------------------------------------------*/
  if (!rc)
    return -1; 
    
  Py_INCREF(PyExc_Warning); 
  Py_INCREF(PyExc_Error); 
  Py_INCREF(PyExc_InterfaceError); 
  Py_INCREF(PyExc_DatabaseError); 
  Py_INCREF(PyExc_DataError); 
  Py_INCREF(PyExc_OperationalError); 
  Py_INCREF(PyExc_IntegrityError); 
  Py_INCREF(PyExc_InternalError); 
  Py_INCREF(PyExc_ProgrammingError); 
  Py_INCREF(PyExc_NotSupportedError); 
  Py_INCREF(PyExcTable); 

  return 0;
}

%}

/******************************************************************************/
/*                                                                            */
/* Module Initialization Code                                                 */
/*                                                                            */
/******************************************************************************/
%init
%{

%}
